# -*- coding: iso_8859_1 -*-

import pyfits
import numpy as np
import os

#----------------------------------------------------------------------------------------
#def getdata(Folder, Imagelist, format=np.float32, text_display='yes'):
#
#--- purpose ---
#to load the image data of a serie of FITS files
#
#--- input parameters ---
#Folder = string, gives the folder where the FITS files are stored
#		if Folder contains non-ASCII characters (such as , , , etc) it must specified as a unicode string, with the prefix "u"
#		examples: "C:\\Images\\", u"C:\\Astroides\\"
#Imagelist = list of strings, each string is the name of one FITS file
#		the function will load all the files with complete paths = Folder + Imagelist[i]
#format = defines the type of the image data values. Can be "np.int16", "np.int32", "np.float32", "np.float64", etc.
#		when not specified, the type takes the default value "np.float32"
#		traditionally amateur CCD images are encoded in "np.int16", so all images are converted to this format before saving by the function astro.save()
#		however it is beneficial to perform the processing with a higher accuracy, e.g. to avoid accumulation of quantisation errors
#		for this reason the default type is float32 (= single precision real numbers)
#text_display = string, if equal to 'yes' (default value) a few informative messages are printed in the console window during execution
#		otherwise, no messages are printed
#
#--- output parameters ---
#result = 3D numpy array containing data for list of images. All images must have the same size
#		result[i][j][k] contains the pixel of the ith image, on jth row (j changes in vertical direction) and kth column (k changes in horizontal direction)
#		remark: the 3D array can contain only a single image, but use of 2D array is then recommended
#		OR
#		= 2D numpy array containing data of a single image
#		result[j][k] contains the pixel of the jth row (j changes in vertical direction) and kth column (k changes in horizontal direction)
#----------------------------------------------------------------------------------------

def getdata(Folder, Imagelist, format=np.float32, text_display='yes'):

	if text_display=='yes':
		print
		print "--- loading FITS data ---"
	#loading images
	for i, sImage in enumerate(Imagelist):
	
		if text_display=='yes':
			print sImage
		hdulist = pyfits.open(Folder + sImage)
			
		if i==0:
			imagedata = hdulist[0].data
			if len(Imagelist)==1:
				imagedata = np.tile(imagedata,(1,1))
			else:
				imagedata = np.tile(imagedata,(len(Imagelist),1,1))
		else:
			imagedata[i,:,:] = hdulist[0].data

	hdulist.close()
		
	return np.array(imagedata, dtype=format)
	

#----------------------------------------------------------------------------------------
#def getheaders(Folder, Imagelist, text_display='yes'):
#
#--- purpose ---
#to load all FITS headers of a serie of FITS images
#
#--- input parameters ---
#Folder = string, gives the folder where the FITS files are stored
#		if Folder contains non-ASCII characters (such as , , , etc) it must specified as a unicode string, with the prefix "u"
#		examples: "C:\\Images\\", u"C:\\Astroides\\"
#Imagelist = list of strings, each string is the name of one FITS file
#		the function will inspect all the files with complete paths = Folder + Imagelist[i]
#text_display = string, if equal to 'yes' (default value) a few informative messages are printed in the console window during execution
#		otherwise, no messages are printed
#
#--- output parameters ---
#result = header information for the image list, in the form of a list (over all images) of lists (over all headers) of 3-element lists
#		result[i] contains all the FITS headers of the (i+1)th image
#		result[i][j] contains the full record of the (j+1)th header of the (i+1)th image
#		result[i][j][0] contains the FITS keyword of the (j+1)th header of the (i+1)th image
#		result[i][j][1] contains the value of the (j+1)th header of the (i+1)th image
#		result[i][j][2] contains the comment of the (j+1)th header of the (i+1)th image
#----------------------------------------------------------------------------------------

def getheaders(Folder, Imagelist, text_display='yes'):
	
	if text_display=='yes':
		print 
		print "--- loading FITS headers ---"
	imageheader = []
	
	#loading images
	for i, sImage in enumerate(Imagelist):
	
		if text_display=='yes':
			print sImage
		hdulist = pyfits.open(Folder + sImage)
		
		cardlist = hdulist[0].header.ascardlist()
		headerlist = []
		for x in cardlist:
			headerlist.append([x.key, x.value, x.comment])

		if i==0:
			imageheader = [headerlist]
		else:
			imageheader.append(headerlist)

	hdulist.close()
	return imageheader

#----------------------------------------------------------------------------------------
#def getheadervalues(Folder, Imagelist, Header):
#
#--- purpose ---
#to load all the values of a specific FITS header for a serie of FITS images
#
#--- input parameters ---
#Folder = string, gives the folder where the FITS files are stored
#		if Folder contains non-ASCII characters (such as , , , etc) it must specified as a unicode string, with the prefix "u"
#		examples: "C:\\Images\\", u"C:\\Astroides\\"
#Imagelist = list of strings, each string is the name of one FITS file
#		the function will inspect all the files with complete paths = Folder + Imagelist[i]
#Header = FITS keyword for which the values must be returned
#
#--- output parameters ---
#result = list containing all values (either integers, floats or strings) of the specified FITS header on the list of images
#		result[i] concerns the (i+1)th image
#----------------------------------------------------------------------------------------

def getheadervalues(Folder, Imagelist, Header):
	
	headervalues = []
	
	#loading images
	for i, sImage in enumerate(Imagelist):
	
		hdulist = pyfits.open(Folder + sImage)
		headervalues.append(hdulist[0].header[Header])

	hdulist.close()
		
	return headervalues

#----------------------------------------------------------------------------------------
#def check_extension(file_name):
#
#--- purpose ---
#to check if a given file has the ".fit" or ".fits" extension
#
#--- input parameters ---
#file_name = name of file to be checked
#
#--- output parameters ---
#result = boolean, True if specified file has the ".fit" or ".fits" extension ; False otherwise
#----------------------------------------------------------------------------------------

def check_extension(file_name):
	ext = os.path.splitext(file_name)[1]
	if (ext==".fit") or (ext==".fits"):
		answer = True
	else:
		answer = False
	return answer