# -*- coding: iso_8859_1 -*-

import numpy as np
import sys
import process, stats, transform

#----------------------------------------------------------------------------------------
#def find_stars(image0, threshold, detection_area, excluded=[], included=[]):
#
#--- purpose ---
#detects the positions of the brightest stars on one image
#stars very close to image boundaries are ignored
#
#--- input parameters ---
#image0 = 2D numpy array containing image data
#		image0[j][k] contains the pixel of the jth row (j changes in vertical direction) and kth column (k changes in horizontal direction)
#threshold = defines the threshold used to detect the stars
#		always between 0 and 1; reasonnable values are 0.05, 0.10
#		when threshold is set to low values, more stars are considered
#		the actual threshold that is used is np.mean(image) + threshold*(np.max(image)-np.mean(image))
#detection_area = defines the +/- size of the square area used for star detection and PSF fitting
#		typical value is detection_area=10, resulting in a square of 21*21 pixels
#excluded = [[x11,x12,y11,y12], ..., [xn1,xn2,yn1,yn2]] defines a list of rectangular zones [xi1,xi2,yi1,yi2] on first image that are excluded from star identifications (e.g. galaxie(s), artefact(s), moving object(s)..)
#included = [[x11,x12,y11,y12], ..., [xn1,xn2,yn1,yn2]] defines a list of rectangular zones [xi1,xi2,yi1,yi2] on first image on which all star identifications must be done 
#		(all parts of the initial image not belonging to at least one of these zones are rejected)
#
#--- output parameters ---
#result = 2D numpy array with 3 columns x, y and maxi
#		- x=result[:,0] and y=result[:,1] are the (x=vertical,y=horizontal) coordinates of the detected star local maximum
#		- maxi=result[:,2] is the star maximum signal in ADU
#		the stars are returned in order of decreasing max intensity
#		result.shape[1] is the number of detected stars
#----------------------------------------------------------------------------------------

def find_stars(image0, threshold, detection_area, excluded=[], included=[]):

	#image sizes
	size_x = image0.shape[0]
	size_y = image0.shape[1]

	#calculating threshold
	mean = np.mean(image0.flatten())
	maxi = np.max(image0.flatten())
	threshold = mean + threshold*(maxi-mean)
	
	#finding potential star areas
	indices = np.where(image0>threshold)
	x = indices[0]
	y = indices[1]
	
	#finding approximate star centers
	stars1 = []
	for i in range(0,len(x)):
		if (x[i]>=detection_area)&(x[i]<size_x-detection_area)&(y[i]>=detection_area)&(y[i]<size_y-detection_area):
			area = image0[x[i]-detection_area:x[i]+detection_area+1,y[i]-detection_area:y[i]+detection_area+1]
			xcenter,ycenter = np.where(area==np.max(np.max(area)))
			stars1.append([xcenter[0]+x[i]-detection_area, ycenter[0]+y[i]-detection_area])

	#removes hot pixels and double occurrences
	stars2 = []
	for center in stars1:
		stars1.remove(center)
		if center in stars1:
			if center not in stars2:
				stars2.append(center)

	#excluded or included area
	if len(excluded)>0:
		for zone in excluded:
			xmin, xmax, ymin, ymax = zone
			for xy in stars2:
				if (xy[0]>=xmin)and(xy[0]<=xmax)and(xy[1]>=ymin)and(xy[1]<=ymax):
					stars2.remove(xy)
	elif len(included)>0:
		for zone in included:
			xmin, xmax, ymin, ymax = zone
			for xy in stars2:
				if (xy[0]<xmin)or(xy[0]>xmax)or(xy[1]<ymin)or(xy[1]>ymax):
					stars2.remove(xy)
				
	#finding accurate star centers
	stars3 = []

	for xy in stars2:
		#print xy
		if (xy[0]>=detection_area)&(xy[0]<size_x-detection_area)&(xy[1]>=detection_area)&(xy[1]<size_y-detection_area):
			p = process.fit_gauss_elliptical([xy[0]-detection_area,xy[1]-detection_area],image0[xy[0]-detection_area:xy[0]+detection_area+1, xy[1]-detection_area:xy[1]+detection_area+1])
			max, floor, height, mean_x, mean_y, fwhm_1, fwhm_2, angle = p
			stars3.append([mean_x, mean_y, max])

	#ordering by decreasing intensity
	stars3 = np.array(stars3)
	stars3 = np.flipud(stars3[np.argsort(stars3[:,2]),:])
		
	return stars3

#----------------------------------------------------------------------------------------
#def find_shift_stars_matching(image1, image2, threshold, detection_area, excluded=[], included=[], text_display='no'):
#
#--- purpose ---
#calculates the non-integer x,y shift between two images of the same star field
#algorithm reports the most likely shift, that allows the greatest number of stars to coincide
#
#--- input parameters ---
#image1 = 2D numpy array containing data of first image
#image2 = 2D numpy array containing data of second image
#		image1[j][k] or image2[j][k] contain the pixel of the jth row (j changes in vertical direction) and kth column (k changes in horizontal direction)
#threshold = defines the threshold used to detect the stars
#		always between 0 and 1; reasonnable values are 0.05, 0.10
#		when threshold is set to low values, more stars are considered
#		the actual threshold that is used is np.mean(image) + threshold*(np.max(image)-np.mean(image))
#detection_area = defines the +/- size of the square area used for star detection with astro.find_stars()
#		typical value is detection_area=10, resulting in a square of 21*21 pixels
#excluded = [[x11,x12,y11,y12], ..., [xn1,xn2,yn1,yn2]] defines a list of rectangular zones [xi1,xi2,yi1,yi2] on first image that are excluded from calculations (e.g. galaxie(s), artefact(s), moving object(s)..)
#included = [[x11,x12,y11,y12], ..., [xn1,xn2,yn1,yn2]] defines a list of rectangular zones [xi1,xi2,yi1,yi2] on first image on which all calculations must be done 
#		(all parts of the initial image not belonging to at least one of these zones are rejected)
#text_display = string, if equal to 'yes' then some information about matching the star shifts between the 2 images is displayed, to check result quality
#		otherwise (default mode), nothing is displayed
#
#--- output parameters ---
#result = list of [dx,dy] 2 elements giving the calculated shifts respectively along the x (vertical) and y (horizontal) axes
#----------------------------------------------------------------------------------------

def find_shift_stars_matching(image1, image2, threshold, detection_area, excluded=[], included=[], text_display='no'):

	step = 2.			#shift difference value below which shifts are assumed to belong to the same translation scenario
	#-------------------------------------------------------------------------------------
	#	finding stars
	#-------------------------------------------------------------------------------------

	stars1 = find_stars(image1, threshold, detection_area, excluded, included)
	stars2 = find_stars(image2, threshold, detection_area, excluded, included)

	if (stars1.shape[0]==0)or(stars2.shape[0]==0):
		raw_input('no stars could be detected, shift will not be calculated.\nPress Enter to close')
		sys.exit()		

	#-------------------------------------------------------------------------------------
	#	looking for x and y shift
	#-------------------------------------------------------------------------------------

	delta_x = np.tile(stars2[:,0],(len(stars1),1)) - np.tile(np.reshape(stars1[:,0],(-1,1)),(1,len(stars2)))
	delta_x = delta_x.flatten()

	delta_y = np.tile(stars2[:,1],(len(stars1),1)) - np.tile(np.reshape(stars1[:,1],(-1,1)),(1,len(stars2)))
	delta_y = delta_y.flatten()

	nb_x = np.tile(delta_x,(len(delta_x),1)) - np.tile(np.reshape(delta_x,(-1,1)),(1,len(delta_x)))
	nb_y = np.tile(delta_y,(len(delta_y),1)) - np.tile(np.reshape(delta_y,(-1,1)),(1,len(delta_y)))
	nb = (np.abs(nb_x)<=step) * (np.abs(nb_y)<=step)
	nb = np.sum(nb,axis=1)
	nb_max = np.max(nb)

	if text_display=='yes':
		print 'shift comparison result ='
		print nb

	#if list(nb).count(nb_max) != nb_max:
	#	raw_input('more than one solution, shift could not be calculated.\nPress Enter to close')
	#	sys.exit()
	
	dx = [delta_x[i] for i in range(len(delta_x)) if nb[i]>nb_max/2.]
	dy = [delta_y[i] for i in range(len(delta_y)) if nb[i]>nb_max/2.]

	return [np.mean(dx), np.mean(dy)]

#----------------------------------------------------------------------------------------
#def find_shift_crosscorrelation(image1, image2):
#
#--- purpose ---
#calculates the non-integer x,y shift between two images of the same star field
#algorithm is based on the cross-correlation of the two images
#if there is a moving object, it cannot be disregarded during the calculation, making this technique less accurate
#
#--- input parameters ---
#image1 = 2D numpy array containing data of first image
#image2 = 2D numpy array containing data of second image
#		image1[j][k] or image2[j][k] contain the pixel of the jth row (j changes in vertical direction) and kth column (k changes in horizontal direction)
#
#--- output parameters ---
#result = list of [dx,dy] 2 elements giving the calculated shifts respectively along the x (vertical) and y (horizontal) axes
#----------------------------------------------------------------------------------------

def find_shift_crosscorrelation(image1, image2):

	size_x = image1.shape[0]
	size_y = image1.shape[1]

	#--- calculates cross-correlation between two images ---
	im1 = np.fft.fft2(image1)
	im2 = np.fft.fft2(image2)
	crosscorr = np.fft.ifft2(im1*np.ma.conjugate(im2))

	#--- finds the position of the max of the modulus of the cross-correlation ---
	modulus = np.fft.fftshift(np.abs(crosscorr))
	dx, dy = np.where(modulus==np.max(modulus.flatten()))
			
	#--- zoom in on max position with scaling factor 100x, finds again the position of the max with better accuracy ---
	modulus = modulus * 10000./stats.get_median(modulus)
	mod2 = transform.scale_Bspline(modulus, [dx, dy], 100., 3)
	dx2, dy2 = np.where(mod2==np.max(mod2.flatten()))

	#--- combines previous results
	dx3 = np.float(dx+(dx2-dx)/100.)
	dy3 = np.float(dy+(dy2-dy)/100.)
	dx4 = size_x/2. - dx3
	dy4 = size_y/2. - dy3
	return [dx4, dy4]

